/* 
 * $Id: command.c 32 2009-06-07 20:11:05Z ths $
 *
 * Keykeriki wireless keyboard sniffer 
 * command parser routine(s)
 *
 * Authors/Copyright:
 *    Thorsten Schroeder <ths  (at)  dev.io>
 *
 * Revision: $Rev: 32 $
 *
 * Switzerland, Berne, 2009/04/22
 * Germany, Berlin, 2009/05/08
 *
 * */
#include <stdio.h>
#include <string.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include "main.h"
#include "sd.h"
#include "command.h"
#include "crack.h"
#include "eeprom.h"
#include "trf7900.h"

extern keykeriki_config config;
extern uint8_t key_chA, key_chB;
extern unsigned long sd_addr;


/* command handler table */
/* command, description of command, command handler */
const keykeriki_cmd_handler PROGMEM cmd_handler[] = {
   {"help", "This may help.", keykeriki_cmd_help},
   {"cfg", "show current config.", keykeriki_cmd_cfg},
   {"rssi", "Toggle rssi output.", keykeriki_cmd_rssi},
   {"reboot", "reboot keykeriki.", keykeriki_cmd_reboot},
   {"reset", "soft reset (reboot) keykeriki.", keykeriki_cmd_reboot},
   {"key", "crypto key manipulation/access", keykeriki_cmd_key},
   {"trf", "wireless receiver manipulation/access (try trf help)", keykeriki_cmd_trf},
   {"sdcard", "sdcard manipulation/access (try sdcard help)", keykeriki_cmd_sdcard},
   {"free", "display free memory", keykeriki_cmd_free},
   {NULL, NULL, NULL}
};

/* modifies the receiver
 * parameters to the trf command are:
 *   + kbd getchan/setchan/savechan/loadchan
 *   + mouse  getchan/setchan/savechan/loadchan
 *   + init
 * */
int8_t keykeriki_cmd_trf(char *cmd, char *data, uint16_t datalen)
{
   int8_t ret = CMD_ERR;

   if(cmd == NULL)
      return(ret);

   while(*cmd == ' ') cmd++;

   if(memcmp(cmd, "kbd", 3) == 0) {

      cmd += 3;
      while(*cmd == ' ') cmd++;

      if(memcmp(cmd, "setchan", 7) == 0) { 
         unsigned int _chan = 0;
         cmd += 7;
         while(*cmd == ' ') cmd++;

         sscanf(cmd, "%1x", &_chan);
         trf7900_setfreq(1, _chan);

      } else if(memcmp(cmd, "getchan", 7) == 0) { 
         cmd += 7;
         while(*cmd == ' ') cmd++;
         printf_P(PSTR("\r\n[+] current kbd channel: %02x\r\n"), trf7900_getfreq_a());

      }

      ret = CMD_OK;

   } else if(memcmp(cmd, "mouse", 5) == 0) {

      cmd += 5;

      while(*cmd == ' ') cmd++;

      if(memcmp(cmd, "setchan", 7) == 0) { 
         unsigned int _chan = 0;
         cmd += 7;
         while(*cmd == ' ') cmd++;

         sscanf(cmd, "%1x", &_chan);
         trf7900_setfreq(2, _chan);

      } else if(memcmp(cmd, "getchan", 7) == 0) { 
         cmd += 7;
         while(*cmd == ' ') cmd++;
         printf_P(PSTR("\r\n[+] current mouse channel: %02x\r\n"), trf7900_getfreq_b());

      }

      ret = CMD_OK;

   } else if(memcmp(cmd, "init", 4) == 0) {

      cmd += 4;
      printf_P(PSTR("\r\n[+] (re)initializing TRF.\r\n"));
      trf7900_init(0);

      ret = CMD_OK;
   } else if(memcmp(cmd, "help", 4) == 0) {

      cmd += 4;

      printf_P(PSTR("\r\n[+] trf help:\r\n"));
      printf_P(PSTR("\t[-] init            (re)initialize trf7900\r\n"));
      printf_P(PSTR("\t[-] kbd setchan     set keyboard channel\r\n"));
      printf_P(PSTR("\t[-] kbd getchan     display current keyboard channel\r\n"));
      printf_P(PSTR("\t[-] mouse setchan   set mouse channel\r\n"));
      printf_P(PSTR("\t[-] mouse getchan   display current mouse channel\r\n"));
      printf_P(PSTR("\t[-] help            guess what\r\n"));

      ret = CMD_OK;
   }

   return(ret);
}

/* modifies the crypto key (1 byte key for MS)
 * parameters to the key command are:
 *   + set
 *   + get
 *   + reset
 * */
int8_t keykeriki_cmd_key(char *cmd, char *data, uint16_t datalen)
{
   int8_t ret = CMD_ERR;

   if(cmd == NULL)
      return(ret);

   while(*cmd == ' ') cmd++;

   if(memcmp(cmd, "set", 3) == 0) {
      unsigned int _key = 0;
      unsigned int _chan = 0;
      cmd += 3;
      sscanf(cmd, "%1x %2x", &_chan, &_key);
      crypto_setkey(_chan, _key);

      ret = CMD_OK;
   } else if(memcmp(cmd, "get", 3) == 0) {
      cmd += 3;
      printf_P(PSTR("r\n[+] current keys: %02x/%02x\r\n"), key_chA, key_chB);
      ret = CMD_OK;
   } else if(memcmp(cmd, "reset", 5) == 0) {
      cmd += 3;
      printf_P(PSTR("r\n[+] reset key.\r\n"));
      key_chA = key_chB = 0; /* XXX also store/reset keys in EEPROM -- ths */

      ret = CMD_OK;
   }

   return(ret);
}

/* functions for single commands. They shall be implemented like the ones
 * below and do the parsing of arguments (arg0), store result in arg1 which
 * length is provided by arg2 */

int8_t keykeriki_cmd_rssi(char *cmd, char *data, uint16_t datalen)
{
   /* toggle output of RSSI */
   config.enable_rssi ^= 1;
   eeprom_busy_wait();
   eeprom_write_byte(&ee_cfg_enable_rssi, config.enable_rssi);
   printf_P(PSTR("\r\n[+] rssi_enable = %d\r\n"), config.enable_rssi);

   return 0;
}

/* print a list of all commands + description */
int8_t keykeriki_cmd_help(char *cmd, char *data, uint16_t datalen)
{
   char *tmp = NULL;
   uint8_t idx = 0 ;
   printf_P(PSTR("\r\n[+] help follows...\r\n"));

   for(;;) {
      
      tmp = (char *)pgm_read_word(&cmd_handler[idx].name);

      if(tmp == NULL)
         break;

      printf_P(PSTR("%16s - "), tmp);

      tmp = (char *)pgm_read_word(&cmd_handler[idx].desc);
      printf_P(PSTR("%s\r\n"), tmp);
      idx++;
   }
   return 0;
}

/* print config */
int8_t keykeriki_cmd_cfg(char *cmd, char *data, uint16_t datalen)
{
   keykeriki_dumpconfig();
   return 0;
}

/* soft reset */
int8_t keykeriki_cmd_reboot(char *cmd, char *data, uint16_t datalen)
{
   void(* rst)(void) = NULL;

   printf_P(PSTR("\r\n[d] keykeriki_cmd_reboot(%s, %d)\r\n"), cmd, datalen);
   printf_P(PSTR("[*] keykeriki reboot\r\n"));
   
   rst();

   /* ho ho ho */
   return 0;
}


/* functions */

int8_t process_cmd(char *line, char *data, uint16_t datalen)
{
   int8_t ret = CMD_ERR;
   char *tmp = NULL;
   int8_t (*f)(char *, char *, uint16_t) = NULL;
   uint8_t idx = 0 ;
   uint8_t len = 0 ;
   uint8_t cmp = 0;

   for(;;) {
      
      tmp = (char *)pgm_read_word(&cmd_handler[idx].name);

      if(tmp == NULL)
         break;

      len = strlen(tmp);

      cmp = memcmp(line, tmp, len);
      if(cmp == 0) {
         line += len;
         f = (void *)pgm_read_word(&cmd_handler[idx].func);
         break;
      }
      idx++;
   }

   if(f!=NULL)
      ret = f(line, data, datalen);

   return(ret);
}

/* SD card access
 * parameters to the sdcard command are:
 *   + help
 *   + getaddr/setaddr
 *   + fsformat
 *   + fsreset
 *   + init
 *   + enable
 *   + disable
 * */
int8_t keykeriki_cmd_sdcard(char *cmd, char *data, uint16_t datalen)
{
   int8_t ret = CMD_ERR;

   while(*cmd == ' ') cmd++;

   if((memcmp(cmd, "help", 4) == 0) || (cmd == NULL)) {

      cmd += 4;

      while(*cmd == ' ') cmd++;

      printf_P(PSTR("\r\n[+] sdcard help:\r\n"));
      printf_P(PSTR("\t[-] getaddr          get current address\r\n"));
      printf_P(PSTR("\t[-] setaddr <addr>   set current sector address\r\n"));
      printf_P(PSTR("\t[-] fsformat         initialize sdcard filesystem\r\n"));
      printf_P(PSTR("\t[-] fsreset          reset sdcard filesystem->setaddr 0\r\n"));
      printf_P(PSTR("\t[-] init             init sdcard/driver\r\n"));
      printf_P(PSTR("\t[-] enable           enable sdcard in current config and at boot\r\n"));
      printf_P(PSTR("\t[-] disable          disable sdcard in current config and at boot\r\n"));
      printf_P(PSTR("\t[-] append           toggle sdcard append mode at startup\r\n"));
      printf_P(PSTR("[+] done\r\n"));


      ret = CMD_OK;

   } else if(memcmp(cmd, "getaddr", 7) == 0) {
      cmd += 7;
      while(*cmd == ' ') cmd++;
      
      printf_P(PSTR("r\n[+] current sector %08lx\r\n"), sd_addr);

      ret = CMD_OK;

   } else if(memcmp(cmd, "setaddr", 7) == 0) {
      uint32_t _addr = 0;
      cmd += 7;
      while(*cmd == ' ') cmd++;
      sscanf(cmd, "%08lx", &_addr );

      if(_addr == 0)
         _addr++;

      /*
      if(_addr % 16)
         _addr -= (_addr % 16); 
      */
      cli();
      sd_addr = _addr;
      sei();
      
      printf_P(PSTR("r\n[+] sdcard setaddr %08lx\r\n"), _addr);
      sd_fs_update_header(_addr, NULL);

      ret = CMD_OK;

   } else if(memcmp(cmd, "fsformat", 8) == 0) {
      cmd += 8;
      while(*cmd == ' ') cmd++;
      printf_P(PSTR("\r\n[+] sdcard fsformat not implemented yet. Try dd if=/dev/zero bs=512 of=... at home.\r\n"));
      // XXX implement filesystem format -- ths

      ret = CMD_OK;

   } else if(memcmp(cmd, "fsreset", 7) == 0) {
      cmd += 7;
      while(*cmd == ' ') cmd++;

      cli();
      sd_addr = 1;
      sei();
      sd_fs_update_header(1, NULL);

      ret = CMD_OK;

   } else if(memcmp(cmd, "init", 4) == 0) {

      cmd += 4;

      cli();
      sd_addr = 1;
      sei();
      sd_fs_update_header(1, NULL);
      sd_init();

      ret = CMD_OK;
   } else if(memcmp(cmd, "enable", 6) == 0) {

      cmd += 6;

      sd_enable(1);
      printf_P(PSTR("r\n[+] sdcard enabled.\r\n"));

      ret = CMD_OK;
   } else if(memcmp(cmd, "disable", 7) == 0) {

      cmd += 7;
      sd_disable(1);
      printf_P(PSTR("r\n[+] sdcard disabled.\r\n"));

      ret = CMD_OK;
   } else if(memcmp(cmd, "append", 7) == 0) {

      uint8_t now = 0;
      cmd += 7;
      now = sd_append();
      printf_P(PSTR("r\n[+] sdcard append toggled. now -> %d.\r\n"), now);

      ret = CMD_OK;
   }

   return(ret);
}

int8_t keykeriki_cmd_free(char *cmd, char *output, uint16_t len)
{
   extern char *__brkval;
   extern unsigned char __heap_start;
   size_t heapend = 0;
   size_t allram = RAMEND;

   if(__brkval != 0)
      heapend=(size_t) __brkval; 
   else
      heapend=(size_t)&__heap_start;

   printf_P( PSTR("\r\nfree: %d/%d - heap: %d\r\n") , SP-heapend, allram, heapend-(size_t)&__heap_start);

   return(0);
}

