/* 
 * $Id: main.h 32 2009-06-07 20:11:05Z ths $
 *
 * Microsoft keyboard specific Miller Decoding for 8-bit microcontrollers
 *
 * Authors/Copyright:
 *    Max Moser <mmo  (at)  remote-exploit.org>
 *    Thorsten Schroeder <ths  (at)  dev.io>
 *
 * Revision: $Rev: 32 $
 *
 * Switzerland, Berne, 2009/04/22
 *
 * */
#ifndef _MAIN_H
#define _MAIN_H

#define KBD_LAYOUT   KBD_US
/* some useful KBD LAYOUT ID...? */
#define KBD_US       0x01

enum __e_kbd_vendor {
   KBD_VENDOR_MS,
   KBD_VENDOR_LT
} kbd_vendor;

enum __e_kbd_model_ms {
   KBD_MODEL_MS1000,
   KBD_MODEL_MS2000
} kbd_model_ms;

/*******************************************************/


/*
 * MILLERA = (400/(1/F_CPU))/PRESCALER
 * */
   /* 14.7456 MHz */
   #define  MILLERA 23
   #define  MILLERB 35
   #define  MILLERC 46
   #define  SDATA   58
   #define  SYNC    69
#if 0
   /* 16 MHz */
   #define  MILLERA 25 /* 1 perion is 23*0,000016 = 400usec - 25 */
   #define  MILLERB 37 /* (MILLERA * 1.5) 37.5  */
   #define  MILLERC 50 /* (MILLERA * 2)  50     */
   #define  SDATA   62 /* (MILLERA 2.5)  62.5   */
   #define  SYNC    75 /* (MILLERA * 3) 75      */
#endif

typedef struct __miller_ctx {
   uint8_t __idx;    /* current (bit) index in data buffer */
   uint8_t __prev;   /* previous bit */
   uint8_t started;  /* Did we identify a sync pattern? */
   uint8_t sync;     /* Did we identify a data start pattern? */
   uint8_t ptype;    /* packet type */
   uint8_t data[8];  /* */
} miller_context;

typedef struct __kk_config {
   uint8_t enable_rssi;     /* print RSSI value */
   uint8_t enable_usbout;   /* toggle usb output */
   uint8_t enable_sd;       /* toggle SD card support/write */
   uint8_t sd_append;       /* toggle SD card write append */
   uint8_t enable_crack;    /* toggle cracking support */
   uint8_t enable_decrypt;  /* toggle decrypting support */
   uint8_t enable_2nd_usart; /* toggle 2nd usart support */
   uint32_t usart0_baud;
   uint32_t usart1_baud;

} keykeriki_config;

#define CMDLEN 255
typedef struct __kk_cmd {
   uint8_t len;   /* current index */
   char line[CMDLEN+2]; /* we use len as index into the buffer. 
                              if len overflows we use this behavior 
                              like a modular arithmetic to calculate 
                              the current index - we wrap to the
                              beginning of the buffer. To ensure NULL
                              termination, we add extra space to our 
                              buffer which will never be touched by 
                              user data. */
   uint8_t lock;  /* lock */
   uint8_t rdone; /* read line done. */
} keykeriki_cmd;

miller_context ctx;
long count;       /* store the ICR (counts since last trigger) */


void ms_miller_decode(miller_context *ctx, uint8_t);
void miller_c(miller_context *);
void miller_b(miller_context *);
void miller_a(miller_context *);
void miller_start(miller_context *);
void miller_stop(miller_context *);

void togglepin(void);
void keykeriki_init(void);
void keykeriki_dumpconfig(void);
 



#endif

