/* 
 * $Id: serial.c 32 2009-06-07 20:11:05Z ths $
 *
 * Microsoft wireless keyboard sniffer - serial io functions
 *
 * Authors/Copyright:
 *    Max Moser <mmo  at  remote-exploit.org>
 *    Thorsten Schroeder <ths  at  dev.io>
 *
 * Revision: $Rev: 32 $
 *
 * Switzerland, Berne, 2009/04/22
 *
 * */


/* 2009/04/16 Thorsten Schroeder <ths  at  dev.io> */

#include "serial.h"
#include <stdio.h>

/* Obacht: XXX This code uses (hard-coded!) the second USART of an Atmel atmega
 * 64+ --ths */
void serial_init(uint8_t usel, uint32_t speed)
{
   if(speed == 0)
      speed = 9600UL;

   if(usel == 1) {
      UCSR1B |= (1<<TXEN); // TX anmachen
      UCSR1B |= (1<<RXEN); // RX anmachen
      UCSR1B |= (1<<RXCIE1); // RX interrupt anmachen
      UBRR1H = (F_CPU / (speed * 16L) - 1) >> 8;
      UBRR1L = (F_CPU / (speed * 16L) - 1) & 0xFF;
      
      fdevopen(serial_write_char, NULL);

   } else if(usel == 0) {
      UCSR0B |= (1<<TXEN); // TX anmachen
      UCSR0B |= (1<<RXEN); // RX anmachen
      UCSR0B |= (1<<RXCIE0); // RX interrupt anmachen
      UBRR0H = (F_CPU / (speed * 16L) - 1) >> 8;
      UBRR0L = (F_CPU / (speed * 16L) - 1) & 0xFF;
   }

	return;
}

uint8_t serial_read_char(void)
{
	while (!(UCSR1A & (1<<RXC)));
	return UDR1;
}

int serial_write_char(char c, FILE *stream)
{
	while (!(UCSR1A & (1<<UDRE)));
	UDR1 = c;

	return 0;
}

#ifdef WITH_SECOND_UART
int serial_write_char_0(char c, FILE *stream)
{
	while (!(UCSR0A & (1<<UDRE)));
	UDR0 = c;

	return 0;
}
#endif

/* write string to serial - string MUST be properly terminated */
void serial_write_str(char *str)
{
	while (*str) {
		serial_write_char(*str++, NULL);
	}
	return;
}
