/* 
 * $Id: serial.h 32 2009-06-07 20:11:05Z ths $
 *
 * Microsoft wireless keyboard sniffer - serial io functions
 *
 * Authors/Copyright:
 *    Max Moser <mmo  (at)  remote-exploit.org>
 *    Thorsten Schroeder <ths  (at)  dev.io>
 *
 * Revision: $Rev: 32 $
 *
 * Switzerland, Berne, 2009/04/22
 *
 * */


#ifndef _UART_H
#define _UART_H

/* includes */

#include <avr/io.h>
#include <stdio.h>


/* definitions */
#if defined (__AVR_ATmega32__)
	#define USR 	UCSRA
	#define UCR 	UCSRB
	#define UBRR 	UBRRL
	#define EICR 	EICRB
#endif

// test this:
#if defined (__AVR_ATmega644__)
	#define USR UCSR0A
	#define UCR UCSR0B
	#define UBRR UBRR0L
	#define EICR EICRB
	#define TXEN TXEN0
	#define RXEN RXEN0
	#define RXCIE RXCIE0
	#define UDR UDR0
	#define UDRE UDRE0
#endif


/* TODO: add >= atmega64 usart selection macros */

#define _8N1    0x86


#ifndef F_CPU
#error "F_CPU war noch nicht definiert"
//#define F_CPU 16000000
//#define F_CPU 14745600UL
#endif

//#define BAUD 38400      /* highest possible baudrate with 16MHz crystal */
#define BAUD 57600
//#define BAUD 115200

#define UBRR_VAL ((F_CPU+BAUD*8)/(BAUD*16)-1)
#define BAUD_REAL (F_CPU/(16*(UBRR_VAL+1)))
#define BAUD_ERROR ((BAUD_REAL*1000)/BAUD)
 
 
#if ((BAUD_ERROR<990) || (BAUD_ERROR>1010))
#warning Baudrate error too high
#endif 

#define UHIGH   (UBRR_VAL >> 8)
#define ULOW    (UBRR_VAL & 0xFF)


/* function prototypes */

void serial_init(uint8_t, uint32_t);
int serial_write_char(char c, FILE *stream);
#ifdef WITH_SECOND_UART
int serial_write_char_0(char c, FILE *stream);
#endif
void serial_write_str(char *str);

#endif
