/* 
 * $Id: trf7900.c 32 2009-06-07 20:11:05Z ths $
 *
 * Keykeriki wireless keyboard sniffer 
 * trf7900 driver routine(s)
 *
 * Authors/Copyright:
 *    Thorsten Schroeder <ths  at  dev.io>
 *
 * Revision: $Rev: 32 $
 *
 * Germany, Berlin, 2009/05/09
 *
 * */
#include <stdio.h>
#include <string.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>


#include "eeprom.h"
#include "main.h"
#include "trf7900.h"
#include "twi.h"

static uint8_t trf7900_addr = 0;
static uint8_t trf7900_mode = 0;
static uint8_t trf7900_res  = 0;
static uint8_t trf7900_freq_a = 0;
static uint8_t trf7900_freq_b = 0;

static uint8_t trf7900_write(unsigned char *);

uint8_t trf7900_getaddr(void) { return trf7900_addr; }
uint8_t trf7900_getmode(void) { return trf7900_mode; }
uint8_t trf7900_getres(void) { return trf7900_res; }
uint8_t trf7900_getfreq_a(void) { return trf7900_freq_a; }
uint8_t trf7900_getfreq_b(void) { return trf7900_freq_b; }

uint8_t trf7900_init(trf_addr_t addr)
{
   unsigned char data = 0;
   printf_P(PSTR("[*] TRF7900 init... "));

   if(addr)
      trf7900_addr = addr;
   else {
      eeprom_busy_wait();
      trf7900_addr = eeprom_read_byte(&ee_trf7900_addr);
   }

   eeprom_busy_wait();
   trf7900_freq_a = eeprom_read_byte(&ee_trf7900_freq_a);
   
   eeprom_busy_wait();
   trf7900_freq_b = eeprom_read_byte(&ee_trf7900_freq_b);

   eeprom_busy_wait();
   trf7900_mode = eeprom_read_byte(&ee_trf7900_mode);

   eeprom_busy_wait();
   trf7900_res = eeprom_read_byte(&ee_trf7900_res);

   trf7900_write(&data);

   printf_P(PSTR("[%02x] done\r\n"), data);
   return(data);
}

static uint8_t trf7900_write(unsigned char *data) 
{
   uint8_t ret = 0;
   if(*data)
      ret=twi_master_write(trf7900_addr, data, 1);
   else {
      *data  = (trf7900_mode << 7 );
      *data |= (trf7900_freq_a & 7) << 4;
      *data |= (trf7900_res & 1) << 2;
      *data |= (trf7900_freq_b & 7);

      ret=twi_master_write(trf7900_addr, data, 1);
   }
   return(ret);

}


/* chan = mouse or keyboard channel (a or b)
 * freq = channel frequency selection 
 * */
uint8_t trf7900_setfreq(uint8_t chan, uint8_t freq)
{
   unsigned char data = 0;
   printf_P(PSTR("[*] TRF7900 setfreq %d for channel %c ... "), 
         freq, (chan==TRF_CHAN_A) ? 'A' : 'B');

   switch(chan) {
      case TRF_CHAN_A:
         trf7900_freq_a = freq;
         eeprom_busy_wait();
         eeprom_write_byte(&ee_trf7900_freq_a, freq);
         break;
      case TRF_CHAN_B:
         trf7900_freq_b = freq;
         eeprom_busy_wait();
         eeprom_write_byte(&ee_trf7900_freq_b, freq);
         break;
      default:
         break;
   }

   trf7900_write(&data);
   printf_P(PSTR("[%02x] done\r\n"), data);

   return(0);
}


/* chan = mouse or keyboard channel (a or b)
 * freq = channel frequency selection 
 * */
uint8_t trf7900_getfreq(uint8_t chan)
{
   uint8_t data = 0;

   printf_P(PSTR("[*] TRF7900 getfreq for channel %c ... "), 
         (chan==TRF_CHAN_A) ? 'A' : 'B');

   switch(chan) {
      case TRF_CHAN_A:
         eeprom_busy_wait();
         data = eeprom_read_byte(&ee_trf7900_freq_a);
         break;
      case TRF_CHAN_B:
         eeprom_busy_wait();
         data = eeprom_read_byte(&ee_trf7900_freq_b);
         break;
      default:
         break;
   }

   printf_P(PSTR("[%02x] done\r\n"), data);

   return(data);
}

