/* 
 * $Id: twi.h 32 2009-06-07 20:11:05Z ths $
 *
 * Parts of this code comes from "roboter forum" without any 
 * copyright/author header/information.
 *
 * Thorsten Schroeder <ths  (at)  dev.io>
 * Germany, Berlin, 2009/04/22
 *
 * $Rev: 32 $
 *
 * */

#ifndef _TWI_H
#define _TWI_H

/* config */

#define TWI_PORT 	PORTD
#define TWI_SCL	PD0
#define TWI_SDA	PD1

#define SLAVE_ADDR	0b1101100

/* I2C Speed limit by TRF7900: 30kbps */
#define TWIFREQ  100000

/* ******************************************** */

#define OP_NOP		0x00
#define OP_PING	0x01
#define OP_PONG	0x02

#define OP_DO_TEST	0x10
#define OP_DO_BLINK	0x11

#define OP_DO_RESET	0x20
#define OP_DO_ALERT	0x30

#define TWI_WRITE			0x00  /* write */
#define TWI_READ			0x01  /* read */

/* status codes */
#define TWI_REP_START	0x10  /* start op has been retransmitted */
#define TWI_START			0x08  /* start op has been send */
#define TWI_ARB_LOST		0x38	/* arbitration lost */
// TWI Master Transmitter status codes                      
#define TWI_MTX_ADR_ACK				0x18  // SLA+W has been tramsmitted and ACK received
#define TWI_MTX_ADR_NACK			0x20  // SLA+W has been tramsmitted and NACK received 
#define TWI_MTX_DATA_ACK			0x28  // Data byte has been tramsmitted and ACK received
#define TWI_MTX_DATA_NACK			0x30  // Data byte has been tramsmitted and NACK received 

// TWI Master Receiver status codes  
#define TWI_MRX_ADR_ACK				0x40  // SLA+R has been tramsmitted and ACK received
#define TWI_MRX_ADR_NACK			0x48  // SLA+R has been tramsmitted and NACK received
#define TWI_MRX_DATA_ACK			0x50  // Data byte has been received and ACK tramsmitted
#define TWI_MRX_DATA_NACK			0x58  // Data byte has been received and NACK tramsmitted

// TWI Slave Transmitter status codes
#define TWI_STX_ADR_ACK				0xA8  // Own SLA+R has been received; ACK has been returned
#define TWI_STX_ADR_ACK_M_ARB_LOST	0xB0  // Arbitration lost in SLA+R/W as Master; own SLA+R has been received; ACK has been returned
#define TWI_STX_DATA_ACK			0xB8  // Data byte in TWDR has been transmitted; ACK has been received
#define TWI_STX_DATA_NACK			0xC0  // Data byte in TWDR has been transmitted; NOT ACK has been received
#define TWI_STX_DATA_ACK_LAST_BYTE	0xC8  // Last data byte in TWDR has been transmitted (TWEA = 0); ACK has been received

// TWI Slave Receiver status codes
#define TWI_SRX_ADR_ACK				0x60  // Own SLA+W has been received ACK has been returned
#define TWI_SRX_ADR_ACK_M_ARB_LOST	0x68  // Arbitration lost in SLA+R/W as Master; own SLA+W has been received; ACK has been returned
#define TWI_SRX_GEN_ACK				0x70  // General call address has been received; ACK has been returned
#define TWI_SRX_GEN_ACK_M_ARB_LOST	0x78  // Arbitration lost in SLA+R/W as Master; General call address has been received; ACK has been returned
#define TWI_SRX_ADR_DATA_ACK		0x80  // Previously addressed with own SLA+W; data has been received; ACK has been returned
#define TWI_SRX_ADR_DATA_NACK		0x88  // Previously addressed with own SLA+W; data has been received; NOT ACK has been returned
#define TWI_SRX_GEN_DATA_ACK		0x90  // Previously addressed with general call; data has been received; ACK has been returned
#define TWI_SRX_GEN_DATA_NACK		0x98  // Previously addressed with general call; data has been received; NOT ACK has been returned
#define TWI_SRX_STOP_RESTART		0xA0  // A STOP condition or repeated START condition has been received while still addressed as Slave

// TWI Miscellaneous status codes
#define TWI_NO_STATE				0x			serial_write_str(tmpbuf);						
  // No relevant state information available; TWINT = 0
#define TWI_BUS_ERROR				0x00  // Bus error due to an illegal START or STOP condition

/* functions */
#ifdef _TWI_MASTER
uint8_t 	twi_master_init(uint32_t );
void 		twi_master_activate_pullups(void);
uint8_t 	twi_master_start(uint8_t , uint8_t );
void 		twi_master_stop(void);
uint8_t 	twi_master_write_byte(uint8_t );
uint8_t 	twi_master_write(uint8_t, unsigned char *, uint8_t );

uint8_t 	twi_master_read_ack(void);
uint8_t 	twi_master_read_nack(void);
uint8_t 	twi_master_read(uint8_t, unsigned char *, uint8_t );
#endif
#ifdef _TWI_SLAVE

uint8_t twi_slave_init(uint8_t ,uint32_t );
uint8_t twi_slave_read_ack(void);
uint8_t twi_slave_read_nack(void);
uint8_t twi_slave_read(unsigned char *, uint8_t );
uint8_t twi_slave_write_byte(uint8_t b);
uint8_t twi_slave_write(unsigned char *, uint8_t );
void twi_slave_stop(void);
uint8_t twi_slave_poll(uint8_t *);
uint8_t twi_process_buffer(uint8_t , unsigned char *);





#endif



#endif
